/***************************************************************************
 *   Copyright (C) 2004-2006 by Jim Campbell                               *
 *   ifpgui@gmail.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qobject.h>
#include <qlistview.h>
#include <qstring.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtimer.h>
#include <qdir.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qdeepcopy.h>
#include <errno.h>

#include "progressdlg.h"
#include "createdirdlg.h"
#include "iointerface.h"
#include "ifpinterface.h"


int IOInterface::internal_norm_ProtocolStr(QString& fileName, QString protoStr)
   {
   int pat_pos;
   QString filestr;

   pat_pos = fileName.find("://");
   if (pat_pos >= 0)
      {
      filestr = fileName.left(pat_pos + 3);
      if (filestr != protoStr)
         {
         return(-1);
         }
      fileName = fileName.remove(0, pat_pos + 3);
      }
   return(IO_INTF_OK);
   }

QString IOInterface::internal_get_ProtocolStr(QString& fileName)
   {
   int pat_pos;
   QString filestr;

   filestr = "";
   pat_pos = fileName.find("://");
   if (pat_pos >= 0)
      {
      filestr = fileName.left(pat_pos + 3);
      }
   return(filestr);
   }

int IOInterface::internal_remove_ProtocolStr(QString& fileName)
   {
   int pat_pos, ret;

   ret = IO_INTF_OK;
   pat_pos = fileName.find("://");
   if (pat_pos >= 0)
      {
      fileName = fileName.remove(0, pat_pos + 3);
      }
   return(ret);
   }

/**
 * Upload the files in local_file_list from the local harddrive to the iRiver directory remoteDest
 * @param local_file_list the list of filenames on the local harddrive.
 * @param remoteDest the iRiver directory; the place to put the uploaded file
 * @return IFP_INTF_OK on success (0).  Otherwise return an error code.
 */
int IOInterface::internal_exec_TransferToiFP(QString& remoteDest, QPtrList<FileInfos> *local_file_list)
   {
   int ret, mbret, freespace;
   int filecount, totalsize, tmpcnt, dir_size;
   FileInfos *fi_temp;
   ifp_progress_info pginfo;
   QString Message, fileName, filestr;
   QFileInfo finfo;

   ret = IO_INTF_OK;

   if (ifp_dh == NULL)
      {
      lastErrString = QObject::tr("iRiver device not connected.");
      return(-1);
      }

   if (local_file_list == NULL)
      {
      return(IO_INTF_OK);
      }

   if (local_file_list->isEmpty())
      {
      return(IO_INTF_OK);
      }

   // remove the protocol info
   ret = internal_norm_ProtocolStr(remoteDest, "ifp://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Remote destination protocol not supported \"%1\"").arg(internal_get_ProtocolStr(remoteDest));
      return(-1);
      }

   filecount = 0;
   totalsize = 0;
   // get the size of the files/directories and the number of files
   for (fi_temp = local_file_list->first(); fi_temp; fi_temp = local_file_list->next())
      {
      // remove the protocol info
      fileName = fi_temp->FileName();
      ret = internal_norm_ProtocolStr(fileName, "file://");
      if (ret != IO_INTF_OK)
         {
         lastErrString = QObject::tr("Local file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(fileName));
         return(-1);
         }
      // get the file info
      if (QFileInfo(fileName).isFile())
         {
         finfo.setFile(fileName);
         fi_temp->setValues(fileName, IO_INTF_FILE, finfo.size());
         totalsize += finfo.size();
         filecount++;
         }
      else if (QFileInfo(fileName).isDir())
         {
         tmpcnt = 0;
         dir_size = local_get_dir_size(fileName, &tmpcnt);
         fi_temp->setValues(fileName, IO_INTF_DIR, dir_size);
         totalsize += dir_size;
         // this only counts as one file for case where
         //this is the only thing being uploaded
         filecount++;// += tmpcnt;
         }
      else
         {
         fi_temp->setValues(fileName, -1, 0);
         }
      }

   freespace = ifpgui_ifp_freespace(&ifp_dev);
   if (freespace < totalsize)
      {
      lastErrString = QObject::tr("There is not enough free space to upload the requested files.");
      showInfoMsgBox(QObject::tr("iRiver Upload File Status"), lastErrString);
      return -ENOMEM;
      }

   // do the transfer
   Message = QObject::tr("Uploading Files to iRiver device  %1").arg(remoteDest);
   pginfo.total_to_transfer = totalsize;
   pginfo.total_transfered  = 0;
   pginfo.pgw = new ProgressWindow(Message);
   if (filecount > 1)
      pginfo.is_batch = true;
   else
      pginfo.is_batch = false;

   Message = QObject::tr("Upload files to iRiver device  %1  ?").arg(remoteDest);
   mbret = showYesNoAllMsgBox(QObject::tr("Uploading files to iRiver"), Message);
   if (mbret == QMessageBox::No)
      return(IO_INTF_USER_CANCEL);

   ret = IO_INTF_OK;
   for (fi_temp = local_file_list->first(); fi_temp; fi_temp = local_file_list->next())
      {
      processingFiles = TRUE;
      fi_temp->setProcessed(FALSE);
      fileName = fi_temp->FileName();
      if (mbret != QMessageBox::YesAll)
         {
         Message = QObject::tr("Upload  %1  to iRiver device  %2  ?").arg(fileName).arg(remoteDest);
         mbret = showYesNoAllMsgBox(QObject::tr("Uploading files to iRiver"), Message);
         }
      lastErrString = "";
      if (fi_temp == NULL)
         {
         // skip
         }
      else if (fi_temp->FileType() == IO_INTF_DIR)
         {
         if ((mbret != QMessageBox::Yes) || (mbret != QMessageBox::YesAll))
            {
            ret = iFPuploadDirectory(fileName, remoteDest, &pginfo);
            fi_temp->setProcessed(TRUE);
            tmpcnt = 0;
            pginfo.total_transfered += ifp_get_dir_size(fileName, &tmpcnt);
            }
         }
      else
         {
         if ((mbret != QMessageBox::Yes) || (mbret != QMessageBox::YesAll))
            {
            ret = iFPuploadFile(fileName, remoteDest, &pginfo);
            fi_temp->setProcessed(TRUE);
            pginfo.total_transfered += fi_temp->FileSize();
            }
         }
      if ((ret != IO_INTF_OK) && (ret != IO_INTF_USER_CANCEL))
         {
         lastErrString = QString(ifpgui_ifp_error_message(ret));
         if (pginfo.pgw != NULL)
            {
            delete pginfo.pgw;
            pginfo.pgw = NULL;
            }
         showInfoMsgBox(QObject::tr("iRiver Upload File Status"), lastErrString);
         break;
         }
      if (ret == IO_INTF_USER_CANCEL)
         {
         break;
         }
      processingFiles = FALSE;
      }
   processingFiles = FALSE;
   if (pginfo.pgw != NULL)
      {
      delete pginfo.pgw;
      pginfo.pgw = NULL;
      }
   processingFiles = TRUE;
   ifp_read_file_tree();
   processingFiles = FALSE;
   return(ret);
   }

/**
 * Download the files in remote_file_list from the iFP device to the local directory localDest
 * @param remote_file_list the list of filenames on the iFP device.
 * @param localDest the local directory; the place to save the downloaded file
 * @return IFP_INTF_OK on success (0).  Otherwise return an error code.
 */
int IOInterface::internal_exec_TransferFromiFP(QString& localDest, QPtrList<FileInfos> *remote_file_list)
   {
   int ret, mbret;
   FileInfos *fi_temp, *fi_info;
   int filecount, totalsize, tmpcnt;
   ifp_progress_info pginfo;
   QString Message, fileName, filestr;

   ret = IO_INTF_OK;
   if (remote_file_list == NULL)
      {
      return(IO_INTF_OK);
      }

   if (remote_file_list->isEmpty())
      {
      return(IO_INTF_OK);
      }

   // remove the protocol info
   ret = internal_norm_ProtocolStr(localDest, "ifp://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Local destination protocol not supported \"%1\"").arg(internal_get_ProtocolStr(localDest));
      return(-1);
      }

   filecount = 0;
   totalsize = 0;
   // get the size of the files/directories and the number of files
   for (fi_temp = remote_file_list->first(); fi_temp; fi_temp = remote_file_list->next())
      {
      // remove the protocol info
      fileName = fi_temp->FileName();
      ret = internal_norm_ProtocolStr(fileName, "ifp://");
      if (ret != IO_INTF_OK)
         {
         lastErrString = QObject::tr("Remote file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(fileName));
         return(-1);
         }
      // get the file info
      if (ifp_get_info(fileName, &fi_info) == IO_INTF_OK)
         {
         // copy data from fi_info into fi_temp
         fi_temp->setValues(fileName, fi_info->FileType(), fi_info->FileSize());

         if (fi_temp == NULL)
            {
            // skip
            }
         else if (fi_temp->FileType() == IO_INTF_FILE)
            {
            filecount++;
            totalsize += fi_temp->FileSize();
            }
         else
            {
            tmpcnt = 0;
            totalsize += ifp_get_dir_size(fi_temp->FileName(), &tmpcnt);
            // this only counts as one file for case where
            //this is the only thing being uploaded
            filecount++;// += tmpcnt;
            }
         }
      else
         {
         // copy data from fi_info into fi_temp
         fi_temp->setValues(fileName, -1, 0);
         }
      }
   // do the transfer
   Message = QObject::tr("Downloading Files from ifp device to  %1").arg(localDest);
   pginfo.total_to_transfer = totalsize;
   pginfo.total_transfered  = 0;
   pginfo.pgw = new ProgressWindow(Message);
   if (filecount > 1)
      pginfo.is_batch = true;
   else
      pginfo.is_batch = false;

   Message = QObject::tr("Download files from iRiver to local directory  %1  ?").arg(localDest);
   mbret = showYesNoAllMsgBox(QObject::tr("Downloading files from iRiver"), Message);
   if (mbret == QMessageBox::No)
      return(IO_INTF_USER_CANCEL);

   ret = IO_INTF_OK;
   for (fi_temp = remote_file_list->first(); fi_temp; fi_temp = remote_file_list->next())
      {
      processingFiles = TRUE;
      fi_temp->setProcessed(FALSE);
      if (mbret != QMessageBox::YesAll)
         {
         Message = QObject::tr("Download  %1  from iRiver device to  %2  ?").arg(fi_temp->FileName()).arg(localDest);
         mbret = showYesNoAllMsgBox(QObject::tr("Downloading files from iRiver"), Message);
         }
      lastErrString = "";
      if (fi_temp == NULL)
         {
         // skip
         }
      else if (fi_temp->FileType() == IO_INTF_DIR)
         {
         if ((mbret != QMessageBox::Yes) || (mbret != QMessageBox::YesAll))
            {
            fileName = fi_temp->FileName();
            ret = iFPdownloadDirectory(fileName, localDest, &pginfo);
            fi_temp->setProcessed(TRUE);
            tmpcnt = 0;
            pginfo.total_transfered += ifp_get_dir_size(fi_temp->FileName(), &tmpcnt);
            }
         }
      else
         {
         if ((mbret != QMessageBox::Yes) || (mbret != QMessageBox::YesAll))
            {
            fileName = fi_temp->FileName();
            ret = iFPdownloadFile(fileName, localDest, &pginfo);
            fi_temp->setProcessed(TRUE);
            pginfo.total_transfered += fi_temp->FileSize();
            }
         }
      if ((ret != IO_INTF_OK) && (ret != IO_INTF_USER_CANCEL))
         {
         lastErrString = QString(ifpgui_ifp_error_message(ret));
         showInfoMsgBox(QObject::tr("Download File Status"), lastErrString);
         break;
         }
      if (ret == IO_INTF_USER_CANCEL)
         {
         break;
         }
      processingFiles = FALSE;
      }
   processingFiles = FALSE;
   delete pginfo.pgw;
   return(ret);
   }

/**
 * FileInterface
 * Functions to implement the local file io routines.
 */
/**
 * Upload the files from the local drive to the iFP device
 * @param file_info_list the list of filenames on the local hard drive.
 * @param destination the iFP destination directory.
 * @return IFP_INTF_OK on success (0).  Otherwise return an error code.
 */
int FileInterface::exec_TransferUpload(QString& destination, QPtrList<FileInfos> *file_info_list)
   {
   return(internal_exec_TransferToiFP(destination, file_info_list));
   }
/**
 * Download the files from iFP to local drive
 * @param file_info_list the list of filenames on the iFP device.
 * @param destination the local destination directory.
 * @return IFP_INTF_OK on success (0).  Otherwise return an error code.
 */
int FileInterface::exec_TransferDownload(QString& destination, QPtrList<FileInfos> *file_info_list)
   {
   return(internal_exec_TransferFromiFP(destination, file_info_list));
   }

int FileInterface::exec_Mkdir(QString& baseDir, QString& newDir)
   {
   int ret;
   QDir dirObj;

   ret = internal_norm_ProtocolStr(baseDir, "file://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Local file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(baseDir));
      return(-1);
      }

   CreateDirWindow *mkdirdlg;
   lastErrString = "";
   mkdirdlg = new CreateDirWindow(QObject::tr("Create a local directory"));
   mkdirdlg->setBaseDir(baseDir);

   ret = mkdirdlg->exec();
   newDir = mkdirdlg->getNewDirName();
   delete mkdirdlg;

   if (ret == QDialog::Accepted)
      {
      ret = dirObj.mkdir(newDir, TRUE);
      if (ret != true)
         {
         lastErrString = QObject::tr("Could not make directory [%1]").arg(newDir);
         return(-1);
         }
      return(IO_INTF_OK);
      }
   return(IO_INTF_USER_CANCEL);
   }

int FileInterface::exec_Rename(const QString& oldName, const QString& newName)
   {
   int ret;
   QString tmp_oldName, tmp_newName;

   tmp_oldName = oldName;
   tmp_newName = newName;

   ret = internal_norm_ProtocolStr(tmp_oldName, "file://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Local file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(tmp_oldName));
      return(-1);
      }
   ret = internal_norm_ProtocolStr(tmp_newName, "file://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Local file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(tmp_newName));
      return(-1);
      }

   QFile file(oldName);
   if (!file.exists())
      {
      lastErrString = QObject::tr("File does not exist:  %1.").arg(oldName);
      return(-ENOENT);
      }
   QDir dir;
   ret = dir.rename(oldName, newName, TRUE);
   if (ret != TRUE)
      {
      lastErrString = QObject::tr("Could not rename  %1  to  %2").arg(oldName).arg(newName);
      return(-1);
      }
   return(IO_INTF_OK);
   }

int FileInterface::exec_Delete(QPtrList<FileInfos> *file_info_list)
   {
   FileInfos *fi_temp;
   int mbret, ret;
   QString Message, filename;

   if (file_info_list == NULL)
      {
      return(IO_INTF_OK);
      }
   if (file_info_list->isEmpty())
      {
      return(IO_INTF_OK);
      }

   Message = QObject::tr("Delete selected files?");
   mbret = showYesNoAllMsgBox(QObject::tr("Delete selected local files and directories?"), Message);
   if (mbret == QMessageBox::No)
      return(IO_INTF_USER_CANCEL);

   for (fi_temp = file_info_list->first(); fi_temp; fi_temp = file_info_list->next())
      {
      lastErrString = "";
      fi_temp->setProcessed(FALSE);
      filename = fi_temp->FileName();
      ret = internal_norm_ProtocolStr(filename, "file://");
      if (ret != IO_INTF_OK)
         {
         lastErrString = QObject::tr("Local file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(filename));
         return(-1);
         }
      if (fi_temp->FileType() == IO_INTF_DIR)
         {
         // Delete Directory
         QDir dir(filename);
         if (dir.count() > 2)
            {
            lastErrString = QObject::tr("Directory not empty  %1").arg(filename);
            }
         else
            {
            if (mbret != QMessageBox::YesAll)
               {
               Message = QObject::tr("Delete local directory  %1?").arg(filename);
               mbret = showYesNoAllMsgBox(QObject::tr("Delete local directory"), Message);
               }
            if ((mbret == QMessageBox::Yes) || (mbret == QMessageBox::YesAll))
               {
               if (!dir.rmdir(filename, TRUE))
                  {
                  lastErrString = QObject::tr("Could not delete directory  %1").arg(filename);
                  }
               else
                  {
                  fi_temp->setProcessed(TRUE);
                  }
               }
            else if (mbret != QMessageBox::NoAll)
               {
               break;
               }
            }
         }
      else // IO_INTF_FILE, IO_INTF_SYMLINK, IO_INTF_SPECIAL
         {
         // Delete File
         if (mbret != QMessageBox::YesAll)
            {
            Message = QObject::tr("Delete local file  %1?").arg(filename);
            mbret = showYesNoAllMsgBox(QObject::tr("Delete local file"), Message);
            }
         if ((mbret == QMessageBox::Yes) || (mbret == QMessageBox::YesAll))
            {
            QFile file(filename);
            if (!file.remove())
               {
               lastErrString = QObject::tr("Could not delete file  %1").arg(filename);
               //lastErrString = file.errorString();
               }
            else
               {
               fi_temp->setProcessed(TRUE);
               }
            }
         else if (mbret != QMessageBox::NoAll)
            {
            break;
            }
         }
      if (lastErrString != "")
         {
         // dispaly error
         showInfoMsgBox(QObject::tr("Delete Status"), lastErrString);
         }
      }
   return(IO_INTF_OK);
   }

int FileInterface::exec_ShowProperties(const QString& fileName)
   {
   if (fileName == "")
      {
      // satisfy compilier
      }
   return(IO_INTF_OK);
   }

int FileInterface::exec_GetDirectoryListing(QString dirName, QPtrList<FileInfos> *file_info_list)
   {
   int ret;
   const QFileInfoList *files;

   if (file_info_list == NULL)
      {
      return(IO_INTF_OK);
      }

   file_info_list->clear();

   // remove the protocol info
   ret = internal_norm_ProtocolStr(dirName, "file://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Local file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(dirName));
      return(-1);
      }

   QDir thisDir(dirName);
   if (!thisDir.isReadable())
      {
      return(IO_INTF_OK);
      }

   processingFiles = TRUE;
   files = thisDir.entryInfoList();
   if (files)
      {
      QFileInfoListIterator it(*files);
      QFileInfo *fi;
      while((fi = it.current()) != 0)
         {
         ++it;
         if (fi->fileName() == "." || fi->fileName() == "..")
            continue; // nothing

         // fill in info
         // check for directories before symlinks, so we catch symlinks to directories
         if (fi->isDir())
            {
            if (fi->isReadable())
               {
               file_info_list->append(new FileInfos(fi->fileName(), IO_INTF_DIR, 0, NULL));
               }
            else
               {
               file_info_list->append(new FileInfos(fi->fileName(), IO_INTF_DIR_LOCKED, 0, NULL));
               }
            }
         else if (fi->isSymLink())
            {
            file_info_list->append(new FileInfos(fi->fileName(), IO_INTF_SYMLINK, 0, NULL));
            }
         else
            {
            if (fi->isFile())
               {
               file_info_list->append(new FileInfos(fi->fileName(), IO_INTF_FILE, fi->size(), NULL));
               }
            else
               {
               file_info_list->append(new FileInfos(fi->fileName(), IO_INTF_SPECIAL, fi->size(), NULL));
               }
            }
         }
      }
   processingFiles = FALSE;
   return(IO_INTF_OK);
   }

bool FileInterface::get_isReadable(QString& dirName)
   {
   int ret;
   // remove the protocol info
   ret = internal_norm_ProtocolStr(dirName, "file://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Local file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(dirName));
      return(false);
      }
   return (QDir(dirName).isReadable());
   }

QString FileInterface::get_LastError()
   {
   return(lastErrString);
   }


/**
 * iFPInterface
 * Functions to implement the file io routines to interact with the iFP device.
 */
int iFPInterface::ifp_connect(bool showError)
   {
   int ret;
   QString message;

   ret = -1;
   if (ifp_dh == NULL)
      {
      ret = initialize_ifp_device();
      if (errorInLibrary)
         {
         message = QObject::tr("The ifplib Library is missing one or more functions.\n"
            "Please get the latest version from http://ifp-driver.sourcefourge.net.\n\n"
            "USING THE INTERNAL STATIC VERSIONS OF THE FUNCTIONS.");
         showInfoMsgBox(QObject::tr("Error"), message);
         }
      }
   else
      {
      ret = IO_INTF_OK;
      }

   if (ret == IO_INTF_OK)
      {
      processingFiles = TRUE;
      if ((ret = ifp_read_file_tree()) != IO_INTF_OK)
         {
         showInfoMsgBox(QObject::tr("Error"), QObject::tr(iFPNotConnectedMsg));
         fprintf(stderr, "Could not communicate with iRiver device.\n");
         fprintf(stderr, "Try unplugging the USB link, powering off the device, and plugging it back in.\n");
         processingFiles = FALSE;
         return(ret);
         }
      processingFiles = FALSE;
      }
   else
      {
      if (showError)
         {
         showInfoMsgBox(QObject::tr("Error"), QObject::tr(iFPNotConnectedMsg));
         fprintf(stderr, "ifp Device NOT found\n");
         }
      }
   return(ret);
   }


/**
 * Upload the files from iFP to local drive
 * @param file_info_list the list of filenames on the iFP device.
 * @param destination the local destination directory.
 * @return IFP_INTF_OK on success (0).  Otherwise return an error code.
 */
int iFPInterface::exec_TransferUpload(QString& destination, QPtrList<FileInfos> *file_info_list)
   {
   return(internal_exec_TransferFromiFP(destination, file_info_list));
   }
/**
 * Download the files from the local drive to the iFP device
 * @param file_info_list the list of filenames on the local hard drive.
 * @param destination the iFP destination directory.
 * @return IFP_INTF_OK on success (0).  Otherwise return an error code.
 */
int iFPInterface::exec_TransferDownload(QString& destination, QPtrList<FileInfos> *file_info_list)
   {
   return(internal_exec_TransferToiFP(destination, file_info_list));
   }

int iFPInterface::exec_Mkdir(QString& baseDir, QString& newDir)
   {
   int ret;
   QDir dirObj;
   QCString qcfilename;

   if (ifp_dh == NULL)
      {
      lastErrString = QObject::tr("iRiver device not connected.");
      return(-1);
      }
   ret = internal_norm_ProtocolStr(baseDir, "ifp://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Remote file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(baseDir));
      return(-1);
      }

   CreateDirWindow *mkdirdlg;

   lastErrString = "";
   mkdirdlg = new CreateDirWindow(QObject::tr("Create a directory on iRiver device."));
   mkdirdlg->setBaseDir(baseDir);

   ret = mkdirdlg->exec();
   newDir = mkdirdlg->getNewDirName();
   delete mkdirdlg;

   if (ret == QDialog::Accepted)
      {
      processingFiles = TRUE;
      slash2backslash(newDir);
      qcfilename = newDir.utf8();
      ret = ifpgui_ifp_mkdir(&ifp_dev, qcfilename);
      if (ret != IO_INTF_OK)
         {
         lastErrString = QString("Could not make directory [%1] on iRiver device.").arg(newDir);
         processingFiles = FALSE;
         return(-1);
         }
      processingFiles = TRUE;
      ifp_read_file_tree();
      processingFiles = FALSE;
      return(IO_INTF_OK);
      }
   processingFiles = FALSE;
   return(IO_INTF_USER_CANCEL);
   }

int iFPInterface::exec_Rename(const QString& oldName, const QString& newName)
   {
   int ret;
   QCString qcfilename_old, qcfilename_new;
   QString tmp_oldName, tmp_newName;

   if (ifp_dh == NULL)
      {
      lastErrString = QObject::tr("iRiver device not connected.");
      return(-1);
      }

   tmp_oldName = oldName;
   tmp_newName = newName;

   ret = internal_norm_ProtocolStr(tmp_oldName, "ifp://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Remote file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(tmp_oldName));
      return(-1);
      }
   ret = internal_norm_ProtocolStr(tmp_newName, "ifp://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Remote file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(tmp_newName));
      return(-1);
      }

   processingFiles = TRUE;
   slash2backslash(tmp_oldName);
   slash2backslash(tmp_newName);
   qcfilename_old = tmp_oldName.utf8();
   qcfilename_new = tmp_newName.utf8();
   ret = ifpgui_ifp_rename(&ifp_dev, qcfilename_old, qcfilename_new);
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Could not rename  %1  to  %2  on iRiver device.").arg(oldName).arg(newName);
      processingFiles = FALSE;
      return(-1);
      }
   processingFiles = TRUE;
   ifp_read_file_tree();
   processingFiles = FALSE;
   return(IO_INTF_OK);
   }

int iFPInterface::exec_Delete(QPtrList<FileInfos> *file_info_list)
   {
   FileInfos *fi_temp;
   int mbret, ret;
   QCString qcfilename;
   QString Message, filename;

   if (ifp_dh == NULL)
      {
      lastErrString = QObject::tr("iRiver device not connected.");
      return(-1);
      }

   if (file_info_list == NULL)
      {
      return(IO_INTF_OK);
      }
   if (file_info_list->isEmpty())
      {
      return(IO_INTF_OK);
      }

   Message = QObject::tr("Delete selected files on iRiver device?");
   mbret = showYesNoAllMsgBox(QObject::tr("Delete selected iRiver files and directories?"), Message);
   if (mbret == QMessageBox::No)
      return(IO_INTF_USER_CANCEL);

   for (fi_temp = file_info_list->first(); fi_temp; fi_temp = file_info_list->next())
      {
      processingFiles = TRUE;
      lastErrString = "";
      fi_temp->setProcessed(FALSE);
      // remove the protocol info
      filename = fi_temp->FileName();
      ret = internal_norm_ProtocolStr(filename, "ifp://");
      if (ret != IO_INTF_OK)
         {
         lastErrString = QObject::tr("Remote file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(filename));
         processingFiles = FALSE;
         return(-1);
         }
      slash2backslash(filename);
      // get the file info
      if (fi_temp->FileType() == IO_INTF_DIR)
         {
         // Delete Directory
         if (mbret != QMessageBox::YesAll)
            {
            Message = QObject::tr("Delete iRiver directory  %1?").arg(filename);
            mbret = showYesNoAllMsgBox(QObject::tr("Delete iRiver directory"), Message);
            }
         if ((mbret == QMessageBox::Yes) || (mbret == QMessageBox::YesAll))
            {
            qcfilename = filename.utf8();
            ret = ifpgui_ifp_delete_dir_recursive(&ifp_dev, qcfilename);
            if (ret != IO_INTF_OK)
               {
               lastErrString = QString(ifpgui_ifp_error_message(ret));
               }
            else
               {
               fi_temp->setProcessed(TRUE);
               }
            }
         else if (mbret != QMessageBox::NoAll)
            {
            break;
            }
         }
      else // IO_INTF_FILE, IO_INTF_SYMLINK, IO_INTF_SPECIAL
         {
         // Delete File
         if (mbret != QMessageBox::YesAll)
            {
            Message = QObject::tr("Delete iRiver file?   %1").arg(filename);
            mbret = showYesNoAllMsgBox(QObject::tr("Delete iRiver file"), Message);
            }
         if ((mbret == QMessageBox::Yes) || (mbret == QMessageBox::YesAll))
            {
            qcfilename = filename.utf8();
            ret = ifpgui_ifp_delete(&ifp_dev, qcfilename);
            if (ret != IO_INTF_OK)
               {
               lastErrString = QString(ifpgui_ifp_error_message(ret));
               }
            else
               {
               fi_temp->setProcessed(TRUE);
               }
            }
         else if (mbret != QMessageBox::NoAll)
            {
            break;
            }
         }
      if (lastErrString != "")
         {
         // dispaly error
         showInfoMsgBox(QObject::tr("iRiver Delete Status"), lastErrString);
         }
      processingFiles = FALSE;
      }
   processingFiles = TRUE;
   ifp_read_file_tree();
   processingFiles = FALSE;
   return(IO_INTF_OK);
   }

int iFPInterface::exec_ShowProperties(const QString& fileName)
   {
   if (ifp_dh == NULL)
      {
      lastErrString = QObject::tr("iRiver device not connected.");
      return(-1);
      }

   if (fileName == "")
      {
      // satisfy compilier
      }
   return(IO_INTF_OK);
   }

int iFPInterface::exec_GetDirectoryListing(QString dirName, QPtrList<FileInfos> *file_info_list)
   {
   int ret;

   if (ifp_dh == NULL)
      {
      ret = ifp_connect(TRUE);
      if (ret != IO_INTF_OK)
         {
         lastErrString = QObject::tr("iRiver device not connected.");
         return(ret);
         }
      }

   if (file_info_list == NULL)
      {
      return(IO_INTF_OK);
      }

   file_info_list->clear();

   // remove the protocol info
   ret = internal_norm_ProtocolStr(dirName, "ifp://");
   if (ret != IO_INTF_OK)
      {
      lastErrString = QObject::tr("Remote file protocol not supported \"%1\"").arg(internal_get_ProtocolStr(dirName));
      return(-1);
      }

   slash2backslash(dirName);
   processingFiles = TRUE;
   ret = iFPgetFileList(dirName, file_info_list);
   processingFiles = FALSE;

   return(ret);
   }

bool iFPInterface::get_isReadable(QString& dirName)
   {
   if (dirName == "")
      {
      // satisfy compilier
      }
   return(true);
   }

QString iFPInterface::get_LastError()
   {
   return(lastErrString);
   }

